#! /bin/bash

# Installation script for Microway InfiniScope (TM)
# Copyright 2014 by Microway, Inc.

# ======================================================================

# Change this if necessary

INSTALL_DIR="/usr/local/microway"

# ======================================================================

# Don't change these.

REQUIRED_USER="root"
GROUP=infiniscope

# ======================================================================

# Greeting

echo
echo "Microway InfiniScope Installation Script, Part 1 (GUI)"
echo

# ======================================================================

# Check that installer is root

if [ "$USER" != "$REQUIRED_USER" ] ; then
  echo You must be logged in as $REQUIRED_USER to install InfiniScope.
  exit 126
fi

# ======================================================================

# Add infiniscope group if not already present, and add users to group

grep -q $GROUP /etc/group
if [ $? -eq 1 ] ; then
  echo Adding group $GROUP
  PATH=$PATH:/usr/sbin:/sbin ; groupadd $GROUP
fi

echo The line for the $GROUP group in /etc/group is now
grep $GROUP /etc/group

read -p "Would you like to add users to the $GROUP group? [yN] "
if [[ "$REPLY" == [yY]* ]] ; then
  while [ 1 ]
    do
      read -p "Username (<Enter> if no more): "
      if [ -z $REPLY ] ; then
        break
      fi
      username=$REPLY
      echo Adding $username to $GROUP
      grep -q ${GROUP}:.*$username /etc/group || sed -i -e "s/\(${GROUP}:.*[^:]$\)/\1,${username}/" -e "s/\(${GROUP}:.*:$\)/\1${username}/" /etc/group
    done
  echo The line for $GROUP in /etc/group is now
  grep $GROUP /etc/group
fi

# ======================================================================

# Make installation directory if necessary

if [ ! -d "$INSTALL_DIR" ] ; then
  if [ -f "$INSTALL_DIR" ] ; then
    echo File $INSTALL_DIR exists, but is not a directory
    exit 65
  else
    echo Making directory $INSTALL_DIR
    mkdir -p $INSTALL_DIR
  fi
fi

# ======================================================================

# Copy files to installation directory

cp is                $INSTALL_DIR
cp *.html            $INSTALL_DIR
cp microway-logo.png $INSTALL_DIR
cp Makefile.iscope   $INSTALL_DIR

cp *.so*             $INSTALL_DIR
install license_manager -o root -m 4755 $INSTALL_DIR

# ======================================================================

# Copy configuration file to installation directory

cname=$INSTALL_DIR/infiniscope.conf
if [ -e $cname ] ; then
  echo
  echo Configuration file $cname already exists.
  read -p "Do you want to keep the existing copy? [Yn] "
  if [[ "$REPLY" == [nN]* ]] ; then
    copy=1
    echo Do you want to back up the existing copy
    read -p "    to $cname.bak? [Yn] "
    if [[ "$REPLY" != [nN]* ]] ; then
      cp $cname $cname.bak
    fi
  else
    copy=0
  fi
else
  copy=1
fi

if [ $copy -eq 1 ] ; then
  cp infiniscope.conf $INSTALL_DIR
fi

# ======================================================================

# Change owner, group, and protection for all files in installation
# directory

chown root:root $INSTALL_DIR
chown root:root $INSTALL_DIR/*

chmod 755  $INSTALL_DIR
chmod 755  $INSTALL_DIR/is
chmod 755  $INSTALL_DIR/*wx*
chmod 755  $INSTALL_DIR/*.so
chmod 644  $INSTALL_DIR/infiniscope.conf
chmod 644  $INSTALL_DIR/*.html
chmod 644  $INSTALL_DIR/microway-logo.png

# ======================================================================

# If a key file does not already exist, put one in /usr/local/microway
# Use the same key file as MPI Link-Checker

keyfile=/usr/local/microway/microway.key
if [ ! -e $keyfile ] ; then
  touch $keyfile
fi
chown root:root $keyfile
chmod 666 $keyfile

# ======================================================================

# Create some files if they don't already exist

if [ ! -e /tmp/.infiniscope.lock ] ; then
  touch /tmp/.infiniscope.lock
fi
chown root:$GROUP /tmp/.infiniscope.lock
chmod 666 /tmp/.infiniscope.lock

if [ ! -e /tmp/infiniscope.log ] ; then
  touch /tmp/infiniscope.log
fi
chown root:$GROUP /tmp/infiniscope.log
chmod 666 /tmp/infiniscope.log

if [ ! -d /tmp/infiniscope ] ; then
  rm -f /tmp/infiniscope
fi
if [ ! -e /tmp/infiniscope ] ; then
  mkdir /tmp/infiniscope
fi
chown root:$GROUP /tmp/infiniscope
chmod 777 /tmp/infiniscope

# ======================================================================
# Reminder

echo
echo "Part 1 (InfiniScope GUI installation) is complete."
echo
read -p "Continue with iscollect and flop installation? Only say yes if the GUI is also the head node. (Y/n)"
if [[ "$REPLY" == [nN] ]] ; then
  echo "You must run the IB collector installer on the head node of the IB cluster."
  echo "You will need to edit infiniscope.conf on the GUI system to configure connecting to it."
  exit 0
fi

# ======================================================================
# Make collect and flop from obfuscated source.

make iscollect
make flop

# ======================================================================
# Copy files to installation directory and change owner, group, and
# protection

chown -R root:root $INSTALL_DIR
chmod 4755 $INSTALL_DIR/license_manager  #fix any damage prev command could have done to license_manager
chmod 755  $INSTALL_DIR
install flop -o root -g root -m 755 $INSTALL_DIR
install iscollect.c -o root -g $GROUP $INSTALL_DIR
install iscollect -o root -g $GROUP -m 4750 $INSTALL_DIR


# ======================================================================
# Modify configuration file for OpenSM subnet manager

osmcfile=/etc/opensm.conf

if [ ! -e $osmcfile ] ; then
 osmcfile=/etc/ofa/opensm.conf
fi

if [ ! -e $osmcfile ] ; then
 echo "Could not find opensm.conf in /etc/ or /etc/ofa/."
 echo "This is not harmful for newer OFED."
 echo "Older OFED will need VERBOSE=\"-D 0x40\" in opensm.conf to function"
else
  grep -q "^VERBOSE=.*-D 0x40" $osmcfile
  if [ $? -eq 0 ] ; then
    echo $osmcfile does not have to be changed
  else
    echo This line in $osmcfile has to be changed:
    grep "^VERBOSE=" $osmcfile

    echo I would like to comment that line out and replace it with:
    comment_line="# -D 0x40 writes out the forwarding tables for Microway InfiniScope."
    desired_line="VERBOSE=\"-D 0x40\""
    echo $desired_line
    read -p "OK? [yN] "

    if [[ "$REPLY" == [yY]* ]] ; then
     lines="$comment_line\n$desired_line"
     sed -i -e "s/^VERBOSE=.*$/#&\n$lines/" $osmcfile
     echo $osmcfile changed
    fi
  fi
fi
# ======================================================================

#handle automatic distribution to compute nodes on Microway clusters
rcom="notfound"
rcpf="notfound"
[ -f /usr/local/bin/rcom-nodes ] && rcom=/usr/local/bin/rcom-nodes
[ -f /usr/local/bin/scom-nodes ] && rcom=/usr/local/bin/scom-nodes
[ -f /usr/local/bin/rcpf-nodes ] && rcpf=/usr/local/bin/rcpf-nodes
[ -f /usr/local/bin/scpf-nodes ] && rcpf=/usr/local/bin/scpf-nodes
if [ $rcpf != "notfound" -a $rcom != "notfound" ] ; then  #Microway utils found, provide option to install to nodes
  read -p "Would you like to automaticaly distribute iscollect and flop to the nodes? (Y/n)" 
  if [[ "$REPLY" == [nN] ]] ; then
    echo "If you want to use multi collector mode, or run the fabric"
    echo "loading program (flop) you must do the following:"
    echo
    echo "Create the installation directory on all nodes, and distribute"
    echo "iscollect and flop to them."
    echo
    echo "iscollect must have owner:group = root:$GROUP and protection (chmod) = 4750"
    echo "on all nodes in order to read and write the InfiniBand port registers."
    echo
    exit 0
  fi
  $rcom mkdir -p $INSTALL_DIR
  $rcpf $INSTALL_DIR/iscollect $INSTALL_DIR
  $rcom chmod 4750 $INSTALL_DIR/iscollect
else
  echo
  echo "Microway cluster scripts (rcom/scom and rcpf/scpf) not found."
  echo "Manual distribution of iscollect and flop to the compute nodes is required"
  echo
  echo "You must do the following to complete the installation:"
  echo
  echo "Create the installation directory on all nodes, and distribute"
  echo "iscollect and flop to them."
  echo
  echo "iscollect must have owner:group = root:$GROUP and protection = 4750"
  echo "on all nodes in order to read and write the InfiniBand port registers."
  echo
fi

